/*******************************************************************************
Copyright © 2023 Sebastian Roith

Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*******************************************************************************/

(function(window) {

function ensureNoInput() {
    if (this.pfeile.length)
        return 'Dein Beispiel passt nicht zur Beschreibung.';
    return true;
}

const LEVELS = [
    {
        "task": "<p>Kann folgende Beschreibung richtig sein? Wenn ja, dann zeichne ein zur Beschreibung passendes Beispiel. Zeichne dabei nur die Kräfte ein. Wenn nein, dann klicke auf <em>Überprüfen</em> ohne eine einzige Kraft eingezeichnet zu haben:</p><p style=\"text-align:center;font-style:italic\">Die resultierende Kraft hat eine andere Richtung als alle auf den Körper wirkenden Kräfte.</p>",
        "shadows": [
            { type: 'image', src: 'szenario.svg', x: 0, y: 0, width: 100, height: 60 }
        ],
        "check": function() {
            if (!this.pfeile.length)
                return 'Falsch: Es gibt durchaus ein zur Beschreinung passendes Beispiel';
            if (this.pfeile.reduce((prev, pfeil) => prev || pfeil.x1 != 50 || pfeil.y1 != 30, false))
                return 'Der Fußpunkt aller Pfeile muss die Kugelmitte sein.';
            const F_res = new Pfeil(0, 0, this.pfeile.reduce((prev, pfeil) => prev + pfeil.deltaX, 0), this.pfeile.reduce((prev, pfeil) => prev + pfeil.deltaY, 0));
            F_res.remove();
            const angle_res = F_res.angle;
            if (this.pfeile.reduce((prev, pfeil) => prev || Math.abs(pfeil.angle - angle_res) < 0.01, false))
                return 'Die Richtung der resultierenden Kraft stimmt in deinem Beispiel mit einer Kraftrichtung überein';
            return true;
        }
    },
    {
        "task": "<p>Kann folgende Beschreibung richtig sein? Wenn ja, dann zeichne ein zur Beschreibung passendes Beispiel. Zeichne dabei nur die Kräfte ein. Wenn nein, dann klicke auf <em>Überprüfen</em> ohne eine einzige Kraft eingezeichnet zu haben:</p><p style=\"text-align:center;font-style:italic\">Auf einen Körper wirken zwei Kräfte mit den Beträgen 2 N und 3 N. Die resultierende Kraft beträgt 0 N.</p>",
        "shadows": [
            { type: 'image', src: 'szenario.svg', x: 0, y: 0, width: 100, height: 60 }
        ],
        "check": ensureNoInput
    },
    {
        "task": "<p>Kann folgende Beschreibung richtig sein? Wenn ja, dann zeichne ein zur Beschreibung passendes Beispiel. Zeichne dabei nur die Kräfte ein. Wenn nein, dann klicke auf <em>Überprüfen</em> ohne eine einzige Kraft eingezeichnet zu haben:</p><p style=\"text-align:center;font-style:italic\">Obwohl auf einen Körper genau zwei Kräfte mit demselben Betrag wirken, ist der Betrag der resultierenden Kraft nicht 0 N.</p>",
        "shadows": [
            { type: 'image', src: 'szenario.svg', x: 0, y: 0, width: 100, height: 60 }
        ],
        "check": function() {
            if (!this.pfeile.length)
                return 'Falsch: Es gibt durchaus ein zur Beschreinung passendes Beispiel';
            if (this.pfeile.reduce((prev, pfeil) => prev || pfeil.x1 != 50 || pfeil.y1 != 30, false))
                return 'Der Fußpunkt aller Pfeile muss die Kugelmitte sein.';
            if (this.pfeile.length != 2)
                return 'Es müssen genau zwei Kräfte auf den Körper wirken';
            if (Math.abs(this.pfeile[0].abs - this.pfeile[1].abs) > 0.01)
                return 'Die Beträge der beiden Kräfte dürfen nicht abweichen';
            const F_res = new Pfeil(0, 0, this.pfeile.reduce((prev, pfeil) => prev + pfeil.deltaX, 0), this.pfeile.reduce((prev, pfeil) => prev + pfeil.deltaY, 0));
            F_res.remove();
            if (F_res.abs < 0.01)
                return 'Der Betrag der resultierenden Kraft darf nicht 0 N sein';
            return true;
        }
    },
    {
        "task": "<p>Kann folgende Beschreibung richtig sein? Wenn ja, dann zeichne ein zur Beschreibung passendes Beispiel. Zeichne dabei nur die Kräfte ein. Wenn nein, dann klicke auf <em>Überprüfen</em> ohne eine einzige Kraft eingezeichnet zu haben:</p><p style=\"text-align:center;font-style:italic\">Auf einen Körper wirken nur Kräfte mit den Beträgen 3 N, 4 N und 5 N, die resultierende Kraft beträgt 0 N.</p>",
        "shadows": [
            { type: 'image', src: 'szenario.svg', x: 0, y: 0, width: 100, height: 60 }
        ],
        "check": function() {
            if (!this.pfeile.length)
                return 'Falsch: Es gibt durchaus ein zur Beschreinung passendes Beispiel';
            if (this.pfeile.reduce((prev, pfeil) => prev || pfeil.x1 != 50 || pfeil.y1 != 30, false))
                return 'Der Fußpunkt aller Pfeile muss die Kugelmitte sein.';
            if (this.pfeile.length != 3)
                return 'Es müssen genau drei Kräfte auf den Körper wirken';
            const betraege = this.pfeile.map(pfeil => Math.round(pfeil.abs / 5 * 100));
            const allowed = [300, 400, 500];
            const notAllowed = betraege.filter(abs => allowed.indexOf(abs) == -1);
            if (notAllowed.length)
                return 'Eine deiner eingezeichneten Kräfte hat einen Betrag von ' + ('' + (notAllowed[0] / 100)).replace('.', ',') + ' N';
            const missing = allowed.filter(abs => betraege.indexOf(abs) == -1);
            if (missing.length)
                return 'Keine deiner eingezeichneten Kräfte hat einen Betrag von ' + (missing[0] / 100) + ' N';
            const F_res = new Pfeil(0, 0, this.pfeile.reduce((prev, pfeil) => prev + pfeil.deltaX, 0), this.pfeile.reduce((prev, pfeil) => prev + pfeil.deltaY, 0));
            F_res.remove();
            if (F_res.abs > 0.01)
                return 'Der Betrag der resultierenden Kraft müsste gleich 0 N sein';
            return true;
        }
    },
    {
        "task": "<p>Kann folgende Beschreibung richtig sein? Wenn ja, dann zeichne ein zur Beschreibung passendes Beispiel. Zeichne dabei nur die Kräfte ein. Wenn nein, dann klicke auf <em>Überprüfen</em> ohne eine einzige Kraft eingezeichnet zu haben:</p><p style=\"text-align:center;font-style:italic\">Auf einen Körper wirken nur Kräfte mit den Beträgen 1 N, 2 N und 5 N, die resultierende Kraft beträgt 0 N.</p>",
        "shadows": [
            { type: 'image', src: 'szenario.svg', x: 0, y: 0, width: 100, height: 60 }
        ],
        "check": ensureNoInput
    }
];

window.karokonstruktionTask = {
    levels: LEVELS
};
})(window);
